/*This file is part of protocolbuffer Tools eclipse plugin. .

    It is free software: you can redistribute it and/or modify
    it under the terms of the GNU LESSER GENERAL PUBLIC LICENSE as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    this software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU lesser General Public License for more details.

    You should have received a copy of the GNU LESSER GENERAL PUBLIC LICENSE
    along with this software.  If not, see <http://www.gnu.org/licenses/>.
*/

package org.schmidmeier.protocol_buffer.editors;

import org.eclipse.jface.text.TextAttribute;
import org.eclipse.jface.text.rules.IRule;
import org.eclipse.jface.text.rules.IWhitespaceDetector;
import org.eclipse.jface.text.rules.IWordDetector;
import org.eclipse.jface.text.rules.MultiLineRule;
import org.eclipse.jface.text.rules.RuleBasedScanner;
import org.eclipse.jface.text.rules.SingleLineRule;
import org.eclipse.jface.text.rules.Token;
import org.eclipse.jface.text.rules.WhitespaceRule;
import org.eclipse.jface.text.rules.WordRule;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Display;

public class Scanner extends RuleBasedScanner {
	
	public Scanner() {
		WordRule rule = new WordRule(new IWordDetector() {
			public boolean isWordStart(char c) {
				return Character.isJavaIdentifierStart(c);
			}

			public boolean isWordPart(char c) {
				return Character.isJavaIdentifierPart(c);
			}
		});
		Token keyword = new Token(new TextAttribute(getColor(KEYWORD), null,
				SWT.BOLD));
		Token comment = new Token(new TextAttribute(getColor(COMMENT)));
		Token string = new Token(new TextAttribute(getColor(STRING)));
		// add tokens for each reserved word
		for (int n = 0; n < Parser.KEYWORDS.length; n++) {
			rule.addWord(Parser.KEYWORDS[n], keyword);
		}
		setRules(new IRule[] { rule, new SingleLineRule("#", null, comment),
				new SingleLineRule("//", null, comment),
				new SingleLineRule("'", "'", string, '\\'),
				new MultiLineRule("/*", "*/", comment),

				new WhitespaceRule(new IWhitespaceDetector() {
					public boolean isWhitespace(char c) {
						return Character.isWhitespace(c);
					}
				}), });
	}

	// TODO Color handling
	public static int COMMENT = SWT.COLOR_DARK_GREEN;
	public static int RESERVED = SWT.COLOR_DARK_BLUE;
	public static int KEYWORD = SWT.COLOR_DARK_RED;// SWT.COLOR_DARK_RED;
	public static int KEYWORD2 = SWT.COLOR_DARK_GREEN;
	public static int KEYWORD3 = SWT.COLOR_DARK_MAGENTA;
	public static int STRING = SWT.COLOR_BLUE;

	public static Color getColor(int color) {
		return Display.getCurrent().getSystemColor(color);
	}
}
